@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,11 +27,9 @@
 #include "../ArchiveFile.hxx"
 #include "../ArchiveVisitor.hxx"
 #include "input/InputStream.hxx"
-#include "input/InputPlugin.hxx"
 #include "fs/Path.hxx"
 #include "util/RefCount.hxx"
-#include "util/Error.hxx"
-#include "util/Domain.hxx"
+#include "util/RuntimeError.hxx"
 
 #include <zzip/zzip.h>
 
@@ -59,24 +57,19 @@ public:
 
 	virtual void Visit(ArchiveVisitor &visitor) override;
 
-	virtual InputStream *OpenStream(const char *path,
-					Mutex &mutex, Cond &cond,
-					Error &error) override;
+	InputStream *OpenStream(const char *path,
+				Mutex &mutex, Cond &cond) override;
 };
 
-static constexpr Domain zzip_domain("zzip");
-
 /* archive open && listing routine */
 
 static ArchiveFile *
-zzip_archive_open(Path pathname, Error &error)
+zzip_archive_open(Path pathname)
 {
 	ZZIP_DIR *dir = zzip_dir_open(pathname.c_str(), nullptr);
-	if (dir == nullptr) {
-		error.Format(zzip_domain, "Failed to open ZIP file %s",
-			     pathname.c_str());
-		return nullptr;
-	}
+	if (dir == nullptr)
+		throw FormatRuntimeError("Failed to open ZIP file %s",
+					 pathname.c_str());
 
 	return new ZzipArchiveFile(dir);
 }
@@ -123,22 +116,19 @@ struct ZzipInputStream final : public InputStream {
 	}
 
 	/* virtual methods from InputStream */
-	bool IsEOF() override;
-	size_t Read(void *ptr, size_t size, Error &error) override;
-	bool Seek(offset_type offset, Error &error) override;
+	bool IsEOF() noexcept override;
+	size_t Read(void *ptr, size_t size) override;
+	void Seek(offset_type offset) override;
 };
 
 InputStream *
 ZzipArchiveFile::OpenStream(const char *pathname,
-			    Mutex &mutex, Cond &cond,
-			    Error &error)
+			    Mutex &mutex, Cond &cond)
 {
 	ZZIP_FILE *_file = zzip_file_open(dir, pathname, 0);
-	if (_file == nullptr) {
-		error.Format(zzip_domain, "not found in the ZIP file: %s",
-			     pathname);
-		return nullptr;
-	}
+	if (_file == nullptr)
+		throw FormatRuntimeError("not found in the ZIP file: %s",
+					 pathname);
 
 	return new ZzipInputStream(*this, pathname,
 				   mutex, cond,
@@ -146,35 +136,30 @@ ZzipArchiveFile::OpenStream(const char *pathname,
 }
 
 size_t
-ZzipInputStream::Read(void *ptr, size_t read_size, Error &error)
+ZzipInputStream::Read(void *ptr, size_t read_size)
 {
 	int ret = zzip_file_read(file, ptr, read_size);
-	if (ret < 0) {
-		error.Set(zzip_domain, "zzip_file_read() has failed");
-		return 0;
-	}
+	if (ret < 0)
+		throw std::runtime_error("zzip_file_read() has failed");
 
 	offset = zzip_tell(file);
 	return ret;
 }
 
 bool
-ZzipInputStream::IsEOF()
+ZzipInputStream::IsEOF() noexcept
 {
 	return offset_type(zzip_tell(file)) == size;
 }
 
-bool
-ZzipInputStream::Seek(offset_type new_offset, Error &error)
+void
+ZzipInputStream::Seek(offset_type new_offset)
 {
 	zzip_off_t ofs = zzip_seek(file, new_offset, SEEK_SET);
-	if (ofs < 0) {
-		error.Set(zzip_domain, "zzip_seek() has failed");
-		return false;
-	}
+	if (ofs < 0)
+		throw std::runtime_error("zzip_seek() has failed");
 
 	offset = ofs;
-	return true;
 }
 
 /* exported structures */
