@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,19 +35,19 @@
 #include "fs/AllocatedPath.hxx"
 #include "fs/Traits.hxx"
 #include "fs/FileSystem.hxx"
-#include "fs/Charset.hxx"
 #include "storage/FileInfo.hxx"
 #include "util/Alloc.hxx"
+#include "util/StringCompare.hxx"
 #include "util/UriUtil.hxx"
-#include "util/Error.hxx"
 #include "Log.hxx"
 
+#include <stdexcept>
+#include <memory>
+
 #include <assert.h>
-#include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <memory>
 
 UpdateWalk::UpdateWalk(EventLoop &_loop, DatabaseListener &_listener,
 		       Storage &_storage)
@@ -57,17 +57,17 @@ UpdateWalk::UpdateWalk(EventLoop &_loop, DatabaseListener &_listener,
 {
 #ifndef WIN32
 	follow_inside_symlinks =
-		config_get_bool(CONF_FOLLOW_INSIDE_SYMLINKS,
+		config_get_bool(ConfigOption::FOLLOW_INSIDE_SYMLINKS,
 				DEFAULT_FOLLOW_INSIDE_SYMLINKS);
 
 	follow_outside_symlinks =
-		config_get_bool(CONF_FOLLOW_OUTSIDE_SYMLINKS,
+		config_get_bool(ConfigOption::FOLLOW_OUTSIDE_SYMLINKS,
 				DEFAULT_FOLLOW_OUTSIDE_SYMLINKS);
 #endif
 }
 
 static void
-directory_set_stat(Directory &dir, const FileInfo &info)
+directory_set_stat(Directory &dir, const StorageFileInfo &info)
 {
 	dir.inode = info.inode;
 	dir.device = info.device;
@@ -77,7 +77,7 @@ inline void
 UpdateWalk::RemoveExcludedFromDirectory(Directory &directory,
 					const ExcludeList &exclude_list)
 {
-	db_lock();
+	const ScopeDatabaseLock protect;
 
 	directory.ForEachChildSafe([&](Directory &child){
 			const auto name_fs =
@@ -98,8 +98,6 @@ UpdateWalk::RemoveExcludedFromDirectory(Directory &directory,
 				modified = true;
 			}
 		});
-
-	db_unlock();
 }
 
 inline void
@@ -128,9 +126,8 @@ UpdateWalk::PurgeDeletedFromDirectory(Directory &directory)
 	     i != end;) {
 		if (!directory_child_is_regular(storage, directory,
 						i->name.c_str())) {
-			db_lock();
+			const ScopeDatabaseLock protect;
 			i = directory.playlists.erase(i);
-			db_unlock();
 		} else
 			++i;
 	}
@@ -140,7 +137,7 @@ UpdateWalk::PurgeDeletedFromDirectory(Directory &directory)
 static bool
 update_directory_stat(Storage &storage, Directory &directory)
 {
-	FileInfo info;
+	StorageFileInfo info;
 	if (!GetInfo(storage, directory.GetPath(), info))
 		return false;
 
@@ -190,23 +187,22 @@ FindAncestorLoop(Storage &storage, Directory *parent,
 inline bool
 UpdateWalk::UpdatePlaylistFile(Directory &directory,
 			       const char *name, const char *suffix,
-			       const FileInfo &info)
+			       const StorageFileInfo &info)
 {
 	if (!playlist_suffix_supported(suffix))
 		return false;
 
 	PlaylistInfo pi(name, info.mtime);
 
-	db_lock();
+	const ScopeDatabaseLock protect;
 	if (directory.playlists.UpdateOrInsert(std::move(pi)))
 		modified = true;
-	db_unlock();
 	return true;
 }
 
 inline bool
 UpdateWalk::UpdateRegularFile(Directory &directory,
-			      const char *name, const FileInfo &info)
+			      const char *name, const StorageFileInfo &info)
 {
 	const char *suffix = uri_get_suffix(name);
 	if (suffix == nullptr)
@@ -219,8 +215,9 @@ UpdateWalk::UpdateRegularFile(Directory &directory,
 
 void
 UpdateWalk::UpdateDirectoryChild(Directory &directory,
-				 const char *name, const FileInfo &info)
-{
+				 const ExcludeList &exclude_list,
+				 const char *name, const StorageFileInfo &info)
+try {
 	assert(strchr(name, '/') == nullptr);
 
 	if (info.IsRegular()) {
@@ -230,24 +227,28 @@ UpdateWalk::UpdateDirectoryChild(Directory &directory,
 					info.inode, info.device))
 			return;
 
-		db_lock();
-		Directory *subdir = directory.MakeChild(name);
-		db_unlock();
+		Directory *subdir;
+		{
+			const ScopeDatabaseLock protect;
+			subdir = directory.MakeChild(name);
+		}
 
 		assert(&directory == subdir->parent);
 
-		if (!UpdateDirectory(*subdir, info))
+		if (!UpdateDirectory(*subdir, exclude_list, info))
 			editor.LockDeleteDirectory(subdir);
 	} else {
 		FormatDebug(update_domain,
 			    "%s is not a directory, archive or music", name);
 	}
+} catch (const std::exception &e) {
+	LogError(e);
 }
 
 /* we don't look at "." / ".." nor files with newlines in their name */
 gcc_pure
 static bool
-skip_path(const char *name_utf8)
+skip_path(const char *name_utf8) noexcept
 {
 	return strchr(name_utf8, '\n') != nullptr;
 }
@@ -255,7 +256,7 @@ skip_path(const char *name_utf8)
 gcc_pure
 bool
 UpdateWalk::SkipSymlink(const Directory *directory,
-			const char *utf8_name) const
+			const char *utf8_name) const noexcept
 {
 #ifndef WIN32
 	const auto path_fs = storage.MapChildFS(directory->GetPath(),
@@ -277,12 +278,10 @@ UpdateWalk::SkipSymlink(const Directory *directory,
 		return false;
 	}
 
-	const char *target_str = target.c_str();
-
-	if (PathTraitsFS::IsAbsolute(target_str)) {
+	if (target.IsAbsolute()) {
 		/* if the symlink points to an absolute path, see if
 		   that path is inside the music directory */
-		const auto target_utf8 = PathToUTF8(target_str);
+		const auto target_utf8 = target.ToUTF8();
 		if (target_utf8.empty())
 			return true;
 
@@ -293,7 +292,7 @@ UpdateWalk::SkipSymlink(const Directory *directory,
 			: !follow_outside_symlinks;
 	}
 
-	const char *p = target_str;
+	const char *p = target.c_str();
 	while (*p == '.') {
 		if (p[1] == '.' && PathTraitsFS::IsSeparator(p[2])) {
 			/* "../" moves to parent directory */
@@ -327,30 +326,34 @@ UpdateWalk::SkipSymlink(const Directory *directory,
 }
 
 bool
-UpdateWalk::UpdateDirectory(Directory &directory, const FileInfo &info)
+UpdateWalk::UpdateDirectory(Directory &directory,
+			    const ExcludeList &exclude_list,
+			    const StorageFileInfo &info)
 {
 	assert(info.IsDirectory());
 
 	directory_set_stat(directory, info);
 
-	Error error;
-	const std::unique_ptr<StorageDirectoryReader> reader(storage.OpenDirectory(directory.GetPath(), error));
-	if (reader.get() == nullptr) {
-		LogError(error);
+	std::unique_ptr<StorageDirectoryReader> reader;
+
+	try {
+		reader.reset(storage.OpenDirectory(directory.GetPath()));
+	} catch (const std::runtime_error &e) {
+		LogError(e);
 		return false;
 	}
 
-	ExcludeList exclude_list;
+	ExcludeList child_exclude_list(exclude_list);
 
 	{
 		const auto exclude_path_fs =
 			storage.MapChildFS(directory.GetPath(), ".mpdignore");
 		if (!exclude_path_fs.IsNull())
-			exclude_list.LoadFile(exclude_path_fs);
+			child_exclude_list.LoadFile(exclude_path_fs);
 	}
 
-	if (!exclude_list.IsEmpty())
-		RemoveExcludedFromDirectory(directory, exclude_list);
+	if (!child_exclude_list.IsEmpty())
+		RemoveExcludedFromDirectory(directory, child_exclude_list);
 
 	PurgeDeletedFromDirectory(directory);
 
@@ -361,7 +364,7 @@ UpdateWalk::UpdateDirectory(Directory &directory, const FileInfo &info)
 
 		{
 			const auto name_fs = AllocatedPath::FromUTF8(name_utf8);
-			if (name_fs.IsNull() || exclude_list.Check(name_fs))
+			if (name_fs.IsNull() || child_exclude_list.Check(name_fs))
 				continue;
 		}
 
@@ -370,13 +373,13 @@ UpdateWalk::UpdateDirectory(Directory &directory, const FileInfo &info)
 			continue;
 		}
 
-		FileInfo info2;
+		StorageFileInfo info2;
 		if (!GetInfo(*reader, info2)) {
 			modified |= editor.DeleteNameIn(directory, name_utf8);
 			continue;
 		}
 
-		UpdateDirectoryChild(directory, name_utf8, info2);
+		UpdateDirectoryChild(directory, child_exclude_list, name_utf8, info2);
 	}
 
 	directory.mtime = info.mtime;
@@ -389,9 +392,11 @@ UpdateWalk::DirectoryMakeChildChecked(Directory &parent,
 				      const char *uri_utf8,
 				      const char *name_utf8)
 {
-	db_lock();
-	Directory *directory = parent.FindChild(name_utf8);
-	db_unlock();
+	Directory *directory;
+	{
+		const ScopeDatabaseLock protect;
+		directory = parent.FindChild(name_utf8);
+	}
 
 	if (directory != nullptr) {
 		if (directory->IsMount())
@@ -400,7 +405,7 @@ UpdateWalk::DirectoryMakeChildChecked(Directory &parent,
 		return directory;
 	}
 
-	FileInfo info;
+	StorageFileInfo info;
 	if (!GetInfo(storage, uri_utf8, info) ||
 	    FindAncestorLoop(storage, &parent, info.inode, info.device))
 		return nullptr;
@@ -410,13 +415,14 @@ UpdateWalk::DirectoryMakeChildChecked(Directory &parent,
 
 	/* if we're adding directory paths, make sure to delete filenames
 	   with potentially the same name */
-	db_lock();
-	Song *conflicting = parent.FindSong(name_utf8);
-	if (conflicting)
-		editor.DeleteSong(parent, conflicting);
+	{
+		const ScopeDatabaseLock protect;
+		Song *conflicting = parent.FindSong(name_utf8);
+		if (conflicting)
+			editor.DeleteSong(parent, conflicting);
 
-	directory = parent.CreateChild(name_utf8);
-	db_unlock();
+		directory = parent.CreateChild(name_utf8);
+	}
 
 	directory_set_stat(*directory, info);
 	return directory;
@@ -432,7 +438,7 @@ UpdateWalk::DirectoryMakeUriParentChecked(Directory &root, const char *uri)
 	while ((slash = strchr(name_utf8, '/')) != nullptr) {
 		*slash = 0;
 
-		if (*name_utf8 == 0)
+		if (StringIsEmpty(name_utf8))
 			continue;
 
 		directory = DirectoryMakeChildChecked(*directory,
@@ -450,7 +456,7 @@ UpdateWalk::DirectoryMakeUriParentChecked(Directory &root, const char *uri)
 
 inline void
 UpdateWalk::UpdateUri(Directory &root, const char *uri)
-{
+try {
 	Directory *parent = DirectoryMakeUriParentChecked(root, uri);
 	if (parent == nullptr)
 		return;
@@ -462,13 +468,17 @@ UpdateWalk::UpdateUri(Directory &root, const char *uri)
 		return;
 	}
 
-	FileInfo info;
+	StorageFileInfo info;
 	if (!GetInfo(storage, uri, info)) {
 		modified |= editor.DeleteNameIn(*parent, name);
 		return;
 	}
 
-	UpdateDirectoryChild(*parent, name, info);
+	ExcludeList exclude_list;
+
+	UpdateDirectoryChild(*parent, exclude_list, name, info);
+} catch (const std::exception &e) {
+	LogError(e);
 }
 
 bool
@@ -480,11 +490,13 @@ UpdateWalk::Walk(Directory &root, const char *path, bool discard)
 	if (path != nullptr && !isRootDirectory(path)) {
 		UpdateUri(root, path);
 	} else {
-		FileInfo info;
+		StorageFileInfo info;
 		if (!GetInfo(storage, "", info))
 			return false;
 
-		UpdateDirectory(root, info);
+		ExcludeList exclude_list;
+
+		UpdateDirectory(root, exclude_list, info);
 	}
 
 	return modified;
