@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,14 @@
 #include "lib/smbclient/Mutex.hxx"
 #include "../InputStream.hxx"
 #include "../InputPlugin.hxx"
-#include "util/StringUtil.hxx"
-#include "util/Error.hxx"
+#include "PluginUnavailable.hxx"
+#include "system/Error.hxx"
+#include "util/StringCompare.hxx"
 
 #include <libsmbclient.h>
 
+#include <stdexcept>
+
 class SmbclientInputStream final : public InputStream {
 	SMBCCTX *ctx;
 	int fd;
@@ -52,12 +55,12 @@ public:
 
 	/* virtual methods from InputStream */
 
-	bool IsEOF() override {
+	bool IsEOF() noexcept override {
 		return offset >= size;
 	}
 
-	size_t Read(void *ptr, size_t size, Error &error) override;
-	bool Seek(offset_type offset, Error &error) override;
+	size_t Read(void *ptr, size_t size) override;
+	void Seek(offset_type offset) override;
 };
 
 /*
@@ -65,90 +68,83 @@ public:
  *
  */
 
-static InputPlugin::InitResult
-input_smbclient_init(gcc_unused const config_param &param, Error &error)
+static void
+input_smbclient_init(gcc_unused const ConfigBlock &block)
 {
-	if (!SmbclientInit(error))
-		return InputPlugin::InitResult::UNAVAILABLE;
+	try {
+		SmbclientInit();
+	} catch (const std::runtime_error &e) {
+		// TODO: use std::throw_with_nested()?
+		throw PluginUnavailable(e.what());
+	}
 
 	// TODO: create one global SMBCCTX here?
 
-	// TODO: evaluate config_param, call smbc_setOption*()
-
-	return InputPlugin::InitResult::SUCCESS;
+	// TODO: evaluate ConfigBlock, call smbc_setOption*()
 }
 
 static InputStream *
 input_smbclient_open(const char *uri,
-		     Mutex &mutex, Cond &cond,
-		     Error &error)
+		     Mutex &mutex, Cond &cond)
 {
 	if (!StringStartsWith(uri, "smb://"))
 		return nullptr;
 
-	const ScopeLock protect(smbclient_mutex);
+	const std::lock_guard<Mutex> protect(smbclient_mutex);
 
 	SMBCCTX *ctx = smbc_new_context();
-	if (ctx == nullptr) {
-		error.SetErrno("smbc_new_context() failed");
-		return nullptr;
-	}
+	if (ctx == nullptr)
+		throw MakeErrno("smbc_new_context() failed");
 
 	SMBCCTX *ctx2 = smbc_init_context(ctx);
 	if (ctx2 == nullptr) {
-		error.SetErrno("smbc_init_context() failed");
+		int e = errno;
 		smbc_free_context(ctx, 1);
-		return nullptr;
+		throw MakeErrno(e, "smbc_init_context() failed");
 	}
 
 	ctx = ctx2;
 
 	int fd = smbc_open(uri, O_RDONLY, 0);
 	if (fd < 0) {
-		error.SetErrno("smbc_open() failed");
+		int e = errno;
 		smbc_free_context(ctx, 1);
-		return nullptr;
+		throw MakeErrno(e, "smbc_open() failed");
 	}
 
 	struct stat st;
 	if (smbc_fstat(fd, &st) < 0) {
-		error.SetErrno("smbc_fstat() failed");
-		smbc_close(fd);
+		int e = errno;
 		smbc_free_context(ctx, 1);
-		return nullptr;
+		throw MakeErrno(e, "smbc_fstat() failed");
 	}
 
 	return new SmbclientInputStream(uri, mutex, cond, ctx, fd, st);
 }
 
 size_t
-SmbclientInputStream::Read(void *ptr, size_t read_size, Error &error)
+SmbclientInputStream::Read(void *ptr, size_t read_size)
 {
 	smbclient_mutex.lock();
 	ssize_t nbytes = smbc_read(fd, ptr, read_size);
 	smbclient_mutex.unlock();
-	if (nbytes < 0) {
-		error.SetErrno("smbc_read() failed");
-		nbytes = 0;
-	}
+	if (nbytes < 0)
+		throw MakeErrno("smbc_read() failed");
 
 	offset += nbytes;
 	return nbytes;
 }
 
-bool
-SmbclientInputStream::Seek(offset_type new_offset, Error &error)
+void
+SmbclientInputStream::Seek(offset_type new_offset)
 {
 	smbclient_mutex.lock();
 	off_t result = smbc_lseek(fd, new_offset, SEEK_SET);
 	smbclient_mutex.unlock();
-	if (result < 0) {
-		error.SetErrno("smbc_lseek() failed");
-		return false;
-	}
+	if (result < 0)
+		throw MakeErrno("smbc_lseek() failed");
 
 	offset = result;
-	return true;
 }
 
 const InputPlugin input_plugin_smbclient = {
