@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,13 +54,6 @@ extern "C" {
 int
 fd_set_cloexec(int fd, bool enable);
 
-/**
- * Wrapper for dup(), which sets the CLOEXEC flag on the new
- * descriptor.
- */
-int
-dup_cloexec(int oldfd);
-
 /**
  * Wrapper for open(), which sets the CLOEXEC flag (atomically if
  * supported by the OS).
@@ -68,13 +61,6 @@ dup_cloexec(int oldfd);
 int
 open_cloexec(const char *path_fs, int flags, int mode);
 
-/**
- * Wrapper for pipe(), which sets the CLOEXEC flag (atomically if
- * supported by the OS).
- */
-int
-pipe_cloexec(int fd[2]);
-
 /**
  * Wrapper for pipe(), which sets the CLOEXEC flag (atomically if
  * supported by the OS).
@@ -85,25 +71,7 @@ pipe_cloexec(int fd[2]);
 int
 pipe_cloexec_nonblock(int fd[2]);
 
-#ifndef WIN32
-
-/**
- * Wrapper for socketpair(), which sets the CLOEXEC flag (atomically
- * if supported by the OS).
- */
-int
-socketpair_cloexec(int domain, int type, int protocol, int sv[2]);
-
-/**
- * Wrapper for socketpair(), which sets the flags CLOEXEC and NONBLOCK
- * (atomically if supported by the OS).
- */
-int
-socketpair_cloexec_nonblock(int domain, int type, int protocol, int sv[2]);
-
-#endif
-
-#ifdef HAVE_LIBMPDCLIENT
+#ifdef ENABLE_LIBMPDCLIENT
 /* Avoid symbol conflict with statically linked libmpdclient */
 #define socket_cloexec_nonblock socket_cloexec_nonblock_noconflict
 #endif
@@ -123,42 +91,6 @@ int
 accept_cloexec_nonblock(int fd, struct sockaddr *address,
 			size_t *address_length_r);
 
-
-#ifndef WIN32
-
-struct msghdr;
-
-/**
- * Wrapper for recvmsg(), which sets the CLOEXEC flag (atomically if
- * supported by the OS).
- */
-ssize_t
-recvmsg_cloexec(int sockfd, struct msghdr *msg, int flags);
-
-#endif
-
-#ifdef HAVE_INOTIFY_INIT
-
-/**
- * Wrapper for inotify_init(), which sets the CLOEXEC flag (atomically
- * if supported by the OS).
- */
-int
-inotify_init_cloexec(void);
-
-#endif
-
-#ifdef USE_EVENTFD
-
-/**
- * Wrapper for eventfd() which sets the flags CLOEXEC and NONBLOCK
- * flag (atomically if supported by the OS).
- */
-int
-eventfd_cloexec_nonblock(unsigned initval, int flags);
-
-#endif
-
 /**
  * Portable wrapper for close(); use closesocket() on WIN32/WinSock.
  */
